#include "config.h"
#include "macros.h"
#include "network.h"
#include <fcntl.h>

#define BYTEP(v, n)	( *(unsigned char *)((&v)+n) )


void socket_set_async(int s, int async)
{
	int flags=0;

	context;

	flags = fcntl(s, F_GETFL);
	if(async)
		flags |= O_NONBLOCK;
	else
		flags &= ~O_NONBLOCK;

	fcntl(s, F_SETFL, flags);
}


struct in_addr *resolve_host(const unsigned char *host)
{
	static struct in_addr addr;
	struct hostent *he;

	context;

	memset(&addr, 0, sizeof(addr));

	/* check if the host is just an IP (eg "1.2.3.4") */
#ifdef HAVE_INET_ATON
	if(inet_aton(host, &addr))
		return &addr;
#else
	/* inet_aton() does not work on SunOS 5.6 - reported by Marcin TROFIMIUK <mt23890 at sgh.waw.pl> */
	addr.s_addr = inet_addr(host);
	if(addr.s_addr != -1 /* INADDR_NONE doesnt work either! */)
		return &addr;
#endif

	/* resolve the hostname */
	if(!(he = gethostbyname(host)))
		return 0;

	memcpy(&addr.s_addr, he->h_addr_list[0], 4);

	return &addr;
}


int ip_match(const unsigned char *mask_str, struct in_addr *addr)
{
	struct in_addr mask;
	unsigned char *cmask;
	unsigned char *caddr;
	int i;

	context;

#ifdef HAVE_INET_ATON
	if(!inet_aton(mask_str, &mask))
		return 0;
#else
	if(inet_addr(mask_str) != -1 /* INADDR_NONE */)
		return 0;
#endif

	cmask = (unsigned char *)&mask.s_addr;
	caddr = (unsigned char *)&addr->s_addr;

	for(i=0;i<4;i++)
	{
		if(cmask[i] != 255)
			if(cmask[i] != caddr[i])
				return 0;

	}

	return 1;
}


struct sockaddr_in *make_sockaddr(const unsigned char *host, int port)
{
	static struct sockaddr_in addr;
	struct in_addr *raddr;

	context;

	raddr = resolve_host(host);
	if(!raddr)
	{
		NOTICE("Could not resolve host %s!", host);
		return 0;
	}

	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = raddr->s_addr;

	return &addr;
}


/* connects to a host */
int tcp_connect(const struct sockaddr_in *addr, int async)
{
	int s;

	context;

	if(!addr)
		return -1;

	s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(s == -1)
	{
		NOTICE("Unable to create socket: %s", ERRMSG());
		return -1;
	}

	socket_set_async(s, async);

	if(connect(s, (struct sockaddr *)addr, sizeof(struct sockaddr_in)) == -1)
	{
		if(async && errno == EINPROGRESS)
			return s;

		NOTICE("Unable to connect: %s", ERRMSG());
		close(s);
		return -1;
	}
	return s;
}


int tcp_listen(const struct sockaddr_in *addr, int backlog)
{
	int s;

	context;

	if(!addr)
		return -1;

	s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(s == -1)
	{
		NOTICE("Unable to create socket: %s", ERRMSG());
		return -1;
	}

	if(bind(s, (struct sockaddr *)addr, sizeof(struct sockaddr_in)) == -1)
	{
		NOTICE("Unable to bind to port %d: %s", htons(addr->sin_port), ERRMSG());
		close(s);
		return -1;
	}

	if(listen(s, backlog) == -1)
	{
		NOTICE("Unable to listen on port %d: %s", htons(addr->sin_port), ERRMSG());
		close(s);
		return -1;
	}

	return s;
}



int poll_sock(int s, int write)
{
	struct timeval tv;
	fd_set fd;
	int n;

	context;

	if(s == -1)
	{
		DEBUG("TRYING TO POLL INVALID SOCKET");
		return -1;
	}

	FD_ZERO(&fd);
	FD_SET(s, &fd);
	tv.tv_sec = 0;
	tv.tv_usec = 0;

	if(write)
		n = select(s+1, NULL, &fd, NULL, &tv);
	else
		n = select(s+1, &fd, NULL, NULL, &tv);

	if(n == -1)
	{
		NOTICE("select() error: %s", ERRMSG());
	}

	return n;
}
